import peasy.*;
PeasyCam cam;
import java.util.LinkedList;

LinkedList<LinkedList<Triangle3D>> triangles = new LinkedList<LinkedList<Triangle3D>>();
LinkedList<LinkedList<Triangle3D>> triangles_solar = new LinkedList<LinkedList<Triangle3D>>();
ArrayList<Plant> plants = new ArrayList<Plant>();
ArrayList<Plant> solar = new ArrayList<Plant>();
float depth = 50; 

LinkedList<Triangle3D> buildRow(int timestamp_id){
  float stripwidth = 10;    
  float height_stretch = 1/5.0;  
  LinkedList<Triangle3D> triangles_row = new LinkedList<Triangle3D>();
  for(int plant_id =0; plant_id < plants.size()-1; plant_id++){
      String fuel =   plants.get(plant_id).fuel;
      float p1_1 = plants.get(plant_id).timeseries.get(timestamp_id)*height_stretch;
      float p1_2 = plants.get(plant_id).timeseries.get(timestamp_id+1)*height_stretch;
      float p2_1 = plants.get(plant_id+1).timeseries.get(timestamp_id)*height_stretch;
      float p2_2 = plants.get(plant_id+1).timeseries.get(timestamp_id+1)*height_stretch;
      Triangle3D t1 = new Triangle3D(   stripwidth*plant_id, -p1_1, 0, 
                                        stripwidth*plant_id, -p1_2, -depth, 
                                        stripwidth*(plant_id+1), -p2_1, 0,  fuel);
      Triangle3D t2 = new Triangle3D(   stripwidth*(plant_id+1),  -p2_1, 0,
                                        stripwidth*plant_id, -p1_2, -depth,
                                        stripwidth*(plant_id+1),  -p2_2, -depth, fuel);
      t1.set_id(plant_id);
      t2.set_id(plant_id);
      t1.display();
      t2.display();
      triangles_row.add(t1);
      triangles_row.add(t2);  
  }
  return triangles_row;
}

LinkedList<Triangle3D> buildRow_solar(int timestamp_id){
  float stripwidth = 90;    
  float height_stretch = 1/10.0/8;  
  LinkedList<Triangle3D> triangles_row = new LinkedList<Triangle3D>();
  for(int plant_id =0; plant_id < solar.size()-1; plant_id++){
      String fuel =   solar.get(plant_id).fuel;      
      float p1_1 = -solar.get(plant_id).timeseries.get(timestamp_id)*height_stretch;
      float p1_2 = -solar.get(plant_id).timeseries.get(timestamp_id+1)*height_stretch;
      float p2_1 = -solar.get(plant_id+1).timeseries.get(timestamp_id)*height_stretch;
      float p2_2 = -solar.get(plant_id+1).timeseries.get(timestamp_id+1)*height_stretch;
      Triangle3D t1 = new Triangle3D(   stripwidth*plant_id, -p1_1, 0, 
                                        stripwidth*plant_id, -p1_2, -depth, 
                                        stripwidth*(plant_id+1), -p2_1, 0, fuel);
      Triangle3D t2 = new Triangle3D(   stripwidth*(plant_id+1),  -p2_1, 0,
                                        stripwidth*plant_id, -p1_2, -depth,
                                        stripwidth*(plant_id+1),  -p2_2, -depth, fuel);
      t1.display();
      t2.display();
      triangles_row.add(t1);
      triangles_row.add(t2);  
  }
  return triangles_row;
}



void buildTriangles(){  
  int n_timestamps = 168;
  int start_timestamps = 0;
  for (int timestamp_id=start_timestamps; timestamp_id < start_timestamps+n_timestamps ; timestamp_id++) {    
     triangles.add(buildRow(timestamp_id));  
  }  
}

void buildTrianglesSolar(){  
  int n_timestamps = 168;
  int start_timestamps = 0;
  for (int timestamp_id=start_timestamps; timestamp_id < start_timestamps+n_timestamps ; timestamp_id++) {    
     triangles_solar.add(buildRow_solar(timestamp_id));  
  }  
}

void loadPlants(){  
    Table data_table = loadTable("esm_259.csv");
    int columns = data_table.getColumnCount();
    for (int column=0; column<columns; column++){
      Plant plant = new Plant();
      plant.set_fuel(data_table.getString(0, column));
      for (int row=1; row<data_table.getRowCount(); row++){
        plant.timeseries.append(data_table.getFloat(row, column));
      }
      plants.add(plant);
    }
  }

void loadSolar(){
  Table data_table = loadTable("wind_solar.csv");
  int columns = data_table.getColumnCount();
  for (int column=0; column<columns; column++){
      Plant plant = new Plant();
      plant.set_fuel(data_table.getString(0, column));
      for (int row=1; row<data_table.getRowCount(); row++){
        plant.timeseries.append(data_table.getFloat(row, column));
      }
      solar.add(plant);
    }
}


void initialize() {    
    loadPlants();
    loadSolar();
    buildTriangles();
    buildTrianglesSolar();
}




void setup() {
  size(1280, 800, P3D);
  //fullScreen(P3D, 0);   
  surface.setResizable(true);
  cam = new PeasyCam(this, 400, -300, -200, 400);
  cam.rotateX(0.6);
  
  initialize();
}



int time = 11;
void draw() {  
  background(250);
  lights();  
  float light_scale = 2;//cos(2*PI*time/24.0)*0.1+0.9;  
  directionalLight(128.0*light_scale, 128.0*light_scale, 128.0*light_scale, 1, 10, 10);  
  fill(127);
  for(int timestamp=0; timestamp<30; timestamp++){
    translate(0,0,-depth*0.9);
    rotateX(0.025);
    LinkedList<Triangle3D> triangle_row = triangles.get(timestamp);    
    checkMouse(triangle_row);
    for(Triangle3D triangle: triangle_row){
      triangle.display();
    }        
  }  
  LinkedList<Triangle3D> triangle_row = triangles.pop();  
  triangles.addLast(triangle_row);
  
  pushMatrix();
  translate(0,-300,1500);
   for(int timestamp=0; timestamp<30; timestamp++){
    translate(0,0,-depth*0.9);
    rotateX(-0.025);
    LinkedList<Triangle3D> triangle_row_solar = triangles_solar.get(timestamp);    
    for(Triangle3D triangle: triangle_row_solar){
      triangle.display();
    }        
  }
  popMatrix();
  
  LinkedList<Triangle3D> triangle_row_solar = triangles_solar.pop();
  triangles_solar.addLast(triangle_row_solar);
  
    time++;
  if (time > 24) time = 0;
  
  
}